// -*- C++ -*-
/*!
 * @file PeriodicTaskBase.h
 * @brief TaskFuncBase TaskFunc PeriodicTaskBase class
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2009
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

#ifndef COIL_PERIODICTASKBASE_H
#define COIL_PERIODICTASKBASE_H

#include <coil/TimeValue.h>
#include <coil/TimeMeasure.h>
#include <coil/Task.h>

namespace coil
{
  /*!
   * @if jp
   *
   * @class TaskFuncBase
   * @brief TaskFuncBase NX
   *
   * @else
   *
   * @class TaskFuncBase
   * @brief TaskFuncBase class
   *
   * @endif
   */
  class TaskFuncBase
  {
  public:
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * fXgN^B
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~TaskFuncBase() {}

    /*!
     * @if jp
     *
     * @brief IuWFNg̊֐spz֐
     *
     * IuWFNg̊֐spz֐B
     *
     * @else
     *
     * @brief Functor
     *
     * Pure virtual function for Functor.
     *
     * @endif
     */
    virtual int operator()() = 0;
  };
  
  /*!
   * @if jp
   *
   * @class TaskFunc
   * @brief TaskFunc ev[gNX
   *
   * @else
   *
   * @class TaskFunc
   * @brief TaskFunc template class
   *
   * @endif
   */
  template <typename T, typename F = int (*)()>
  class TaskFunc
    : public TaskFuncBase
  {
  public:
    /*!
     * @if jp
     *
     * @brief RXgN^
     *
     * RXgN^B
     *
     * @param obj IuWFNg
     * @param func ֐
     *
     * @else
     *
     * @brief Constructor
     *
     * Constructor
     *
     * @param obj Object.
     * @param func Function.
     *
     * @endif
     */
    TaskFunc(T* obj, F func)
      : m_obj(obj), m_func(func)
    {
    }

    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * fXgN^B
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~TaskFunc() {}

    /*!
     * @if jp
     *
     * @brief IuWFNg̊֐s
     *
     * IuWFNg̊֐sB
     *
     * @else
     *
     * @brief Functor
     *
     * Execute a function of the object.
     *
     * @endif
     */
    virtual int operator()()
    {
      return (m_obj->*m_func)();
    }

    /*!
     * @if jp
     * @brief IuWFNg
     * @else
     * @brief object
     * @endif
     */
    T* m_obj;

    /*!
     * @if jp
     * @brief ֐
     * @else
     * @brief function
     * @endif
     */
    F m_func;
  };

  /*!
   * @if jp
   *
   * @class PeriodicTaskBase
   * @brief PeriodicTaskBase NX
   *
   * @else
   *
   * @class PeriodicTaskBase
   * @brief PeriodicTaskBase class
   *
   * @endif
   */
  class PeriodicTaskBase
    : public coil::Task
  {
  public:
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * fXgN^B
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~PeriodicTaskBase(){};

    /*!
     * @if jp
     *
     * @brief ^XNsJn鏃z֐
     *
     * ^XNsJn鏃z֐B
     *
     * @else
     *
     * @brief Starting the task
     *
     * Pure virtual function for starting the task.
     *
     * @endif
     */
    virtual void activate() = 0;

    /*!
     * @if jp
     *
     * @brief ^XNsI鏃z֐
     *
     * ^XNsI鏃z֐B
     *
     * @else
     *
     * @brief Finalizing the task
     *
     * Pure virtual function for finalizing the task.
     *
     * @endif
     */
    virtual void finalize() = 0;

    /*!
     * @if jp
     *
     * @brief ^XNs𒆒f鏃z֐
     *
     * ^XNs𒆒f鏃z֐B
     *
     * @else
     *
     * @brief Suspending the task
     *
     * Pure virtual function for suspending the task.
     *
     * @endif
     */
    virtual int suspend(void) = 0;

    /*!
     * @if jp
     *
     * @brief fĂ^XNĊJ鏃z֐
     *
     * fĂ^XNĊJ鏃z֐B
     *
     * @else
     *
     * @brief Resuming the suspended task
     *
     * Pure virtual function for resuming the suspended task.
     *
     * @endif
     */
    virtual int resume(void) = 0;

    /*!
     * @if jp
     *
     * @brief fĂ^XN1s鏃z֐
     *
     * fĂ^XN1s鏃z֐B
     *
     * @else
     *
     * @brief Executing the suspended task one tick
     *
     * Pure virtual function for executing the suspended task one tick.
     *
     * @endif
     */
    virtual void signal() = 0;

    /*!
     * @if jp
     *
     * @brief ^XNs֐Zbg鏃z֐
     *
     * ^XNs֐Zbg鏃z֐B
     *
     * @param func ֐
     * @param delete_in_dtor 폜tO
     *
     * @else
     *
     * @brief Setting task execution function
     *
     * Pure virtual function for setting task execution function.
     *
     * @param func Function.
     * @param delete_in_dtor Delete flag.
     *
     * @endif
     */
    virtual bool setTask(TaskFuncBase* func, bool delete_in_dtor = true) = 0;

    /*!
     * @if jp
     *
     * @brief ^XNs֐Zbg
     *
     * ^XNs֐Zbg
     *
     * @param obj IuWFNg
     * @param fun ֐
     *
     * @return true: , false: s
     *
     * @else
     *
     * @brief Setting task execution function
     *
     * Pure virtual function for setting task execution function.
     *
     * @param obj Object.
     * @param fun Function.
     *
     * @return true: successful, false: failed
     *
     * @endif
     */
    template <class O, class F>
    bool setTask(O* obj, F fun)
    {
      return this->setTask(new TaskFunc<O, F>(obj, fun));
    }

    /*!
     * @if jp
     *
     * @brief ^XNsZbg鏃z֐
     *
     * ^XNsZbg鏃z֐B
     *
     * @param period s
     *
     * @else
     *
     * @brief Setting task execution period
     *
     * Pure virtual function for setting task execution period.
     *
     * @param period Execution period.
     *
     * @endif
     */
    virtual void setPeriod(double period) = 0;

    /*!
     * @if jp
     *
     * @brief ^XNsZbg鏃z֐
     *
     * ^XNsZbg鏃z֐B
     *
     * @param period s
     *
     * @else
     *
     * @brief Setting task execution period
     *
     * Pure virtual function for setting task execution period.
     *
     * @param period Execution period.
     *
     * @endif
     */
    virtual void setPeriod(coil::TimeValue& period) = 0;

    /*!
     * @if jp
     *
     * @brief ^XN֐sԌvLɂ鏃z֐
     *
     * ^XN֐sԌvLɂ鏃z֐B
     *
     * @param value tO(true: L, false: )
     *
     * @else
     *
     * @brief Validate a Task execute time measurement
     *
     * Pure virtual function for validate a Task execute time measurement.
     *
     * @param value flag(true: Valid, false: Invalid).
     *
     * @endif
     */
    virtual void executionMeasure(bool value) = 0;

    /*!
     * @if jp
     *
     * @brief ^XN֐sԌvpz֐
     *
     * ^XN֐sԌvpz֐B
     *
     * @param n v
     *
     * @else
     *
     * @brief Task execute time measurement period
     *
     * Pure virtual function for task execute time measurement period.
     *
     * @param n Measurement period.
     *
     * @endif
     */
    virtual void executionMeasureCount(int n) = 0;

    /*!
     * @if jp
     *
     * @brief ^XNԌvLɂ鏃z֐
     *
     * ^XNԌvLɂ鏃z֐B
     *
     * @param value tO(true: L, false: )
     *
     * @else
     *
     * @brief Validate a Task period time measurement
     *
     * Pure virtual function for validate a Task period time measurement.
     *
     * @param value flag(true: Valid, false: Invalid).
     *
     * @endif
     */
    virtual void periodicMeasure(bool value) = 0;

    /*!
     * @if jp
     *
     * @brief ^XNԌvpz֐
     *
     * ^XNԌvpz֐B
     *
     * @param n v
     *
     * @else
     *
     * @brief Task period time measurement count
     *
     * Pure virtual function for task period time measurement count.
     *
     * @param n Measurement period.
     *
     * @endif
     */
    virtual void periodicMeasureCount(int n) = 0;

    /*!
     * @if jp
     *
     * @brief ^XN֐sԌvʂ擾鏃z֐
     *
     * ^XN֐sԌvʂ擾鏃z֐B
     *
     * @else
     *
     * @brief Get a result in task execute time measurement
     *
     * Pure virtual function for get a result in task execute time measurement.
     *
     * @endif
     */
    virtual coil::TimeMeasure::Statistics getExecStat() = 0;

    /*!
     * @if jp
     *
     * @brief ^XNԌvʂ擾鏃z֐
     *
     * ^XNԌvʂ擾鏃z֐B
     *
     * @else
     *
     * @brief Get a result in task period time measurement
     *
     * Pure virtual function for get a result in task period time measurement.
     *
     * @endif
     */
    virtual coil::TimeMeasure::Statistics getPeriodStat() = 0;

  };
}; // namespace coil

#endif // COIL_PERIODICTASKBASE_H
