// -*- C++ -*-
/*!
 * @file SharedMemory_win32.cpp
 * @brief SharedMemory class
 * @date $Date$
 * @author Nobuhiko Miyamoto <n-miyamoto@aist.go.jp>
 *
 * Copyright (C) 2017 Nobuhiko Miyamoto
 *     Robot Innovation Research Center,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */


#include <coil/SharedMemory.h>
//#include "SharedMemory.h"
#include <string.h>


namespace coil
{
  /*!
   * @if jp
   * @brief RXgN^
   * @else
   * @brief Constructor
   * @endif
   */
  SharedMemory::SharedMemory()
    : m_memory_size(0),
      m_shm(NULL),
      m_handle(NULL)
  {
  }


  /*!
   * @if jp
   * @brief fXgN^
   * @else
   * @brief Destructor
   * @endif
   */
  SharedMemory::~SharedMemory()
  {
    close();
  }

  /*!
   * @if jp
   * @brief Rs[RXgN^
   * @else
   * @brief Copy Constructor
   * @endif
   */
  SharedMemory::SharedMemory(const SharedMemory& rhs)
  {
    m_memory_size = rhs.m_memory_size;
    m_shm_address = rhs.m_shm_address;
    m_shm = rhs.m_shm;
    m_handle = rhs.m_handle;
  }

  /*!
   * @if jp
   * @brief Zq
   * @else
   * @brief Assignment operator
   * @endif
   */
  SharedMemory& SharedMemory::operator=(const SharedMemory& rhs)
  {
    SharedMemory tmp(rhs);
    std::swap(this->m_memory_size, tmp.m_memory_size);
    std::swap(this->m_shm_address, tmp.m_shm_address);
    std::swap(this->m_shm, tmp.m_shm);
    std::swap(this->m_handle, tmp.m_handle);
    return *this;
  }

  /*!
   * @if jp
   *
   * @brief L̐
   *B
   *
   * @param shm_address L̎ʎq
   * @param memory_size L̃TCY
   *
   * @return 0: , -1: s
   *
   * @else
   *
   * @brief Create Shared Memory 
   *
   *
   * @param shm_address 
   * @param memory_size 
   *
   * @return 0: successful, -1: failed
   *
   * @endif
   */
  int SharedMemory::create(std::string shm_address,
                     int memory_size)
  {

    m_shm_address = shm_address;
    m_memory_size = memory_size;

    DWORD highsize = static_cast<DWORD>((m_memory_size >> 32) & 0xFFFFFFFF);
    DWORD lowsize = static_cast<DWORD>(m_memory_size & 0xFFFFFFFF);

    m_handle = CreateFileMapping(
		(HANDLE)-1,
		NULL,
		PAGE_READWRITE | SEC_COMMIT,
		highsize, lowsize, 
		shm_address.c_str());
    /*
    if(GetLastError() == ERROR_ALREADY_EXISTS)
    {
    }
    else
    {
    	
    }
    */

    m_shm = (char *)MapViewOfFile(m_handle, FILE_MAP_ALL_ACCESS, 0, 0, 0);
    return 0;
  }

  /*!
   * @if jp
   *
   * @brief Lւ̃ANZX
   *B
   *
   * @param shm_address L̎ʎq
   *
   * @return 0: , -1: s
   *
   * @else
   *
   * @brief Open Shared Memory 
   *
   *
   * @param shm_address 
   * @param memory_size 
   *
   * @return 0: successful, -1: failed
   *
   * @endif
   */
  int SharedMemory::open(std::string shm_address, int memory_size)
  {
    m_shm_address = shm_address;
	m_memory_size = memory_size;
    m_handle = OpenFileMapping(FILE_MAP_ALL_ACCESS, FALSE, m_shm_address.c_str());
    m_shm = (char *)MapViewOfFile(m_handle, FILE_MAP_ALL_ACCESS, 0, 0, 0);
    return 0;
  }

  /*!
   * @if jp
   *
   * @brief Lւ̏
   *B
   *
   * @param data ރf[^
   *
   * @return 0: , -1: s
   *
   * @else
   *
   * @brief Write Shared Memory 
   *
   *
   * @param data 
   *
   * @return 0: successful, -1: failed
   *
   * @endif
   */
  int SharedMemory::write(const char *data, const int pos, const int size)
  {
	  if (!created())
	  {
		  return -1;
	  }

	  if (m_memory_size < size + pos)
	  {
		  return -1;
	  }

	  memcpy(&m_shm[pos],&data[0],size);
    
	  return 0;
  }

  /*!
   * @if jp
   *
   * @brief L̓ǂݍ
   *B
   *
   * @param data ǂݍރf[^
   *
   * @return 0: , -1: s
   *
   * @else
   *
   * @brief Read Shared Memory 
   *
   *
   * @param data 
   *
   * @return 0: successful, -1: failed
   *
   * @endif
   */
  int SharedMemory::read(char* data, const int pos, const int size)
  {
	  if (!created())
	  {
		  return -1;
	  }

	  memcpy(&data[0],&m_shm[pos],size);
	  return 0;
  }

  /*!
   * @if jp
   *
   * @brief L̍폜
   *B
   *
   *
   * @return 0: , -1: s
   *
   * @else
   *
   * @brief Close Shared Memory 
   *
   *
   *
   * @return 0: successful, -1: failed
   *
   * @endif
   */
  int SharedMemory::close()
  {
    
    if (created())
    {
        UnmapViewOfFile(m_shm);
    }
	else
	{
		return -1;
	}
    if(m_handle != NULL)
    {
    	if (CloseHandle(m_handle) == 0)
    	{
    		return -1;
    	}
	else
	{
		m_handle = NULL;
		return 0;
	}
    }
    return 0;

  }
  /*!
   * @if jp
   *
   * @brief ̃TCY擾
   *B
   *
   *
   * @return TCY
   *
   * @else
   *
   * @brief 
   *
   *
   *
   * @return size
   *
   * @endif
   */
  unsigned int SharedMemory::get_size()
  {
	return m_memory_size;
  }
  /*!
   * @if jp
   *
   * @brief ̃AhX擾
   *B
   *
   *
   * @return AhX
   *
   * @else
   *
   * @brief 
   *
   *
   *
   * @return address
   *
   * @endif
   */
  std::string SharedMemory::get_addresss()
  {
	return m_shm_address;
  };
  /*!
   * @if jp
   *
   * @brief ̎擾
   *
   *
   *
   * @return 
   *
   * @else
   *
   * @brief 
   *
   *
   *
   * @return memory
   *
   * @endif
   */
  char *SharedMemory::get_data()
  {
	return m_shm;
  }



  /*!
   * @if jp
   *
   * @brief t@C폜(Windows̏ꍇ͉Ȃ)
   *
   *
   *
   * @return 0: , -1: s
   *
   * @else
   *
   * @brief 
   *
   *
   *
   * @return 0: successful, -1: failed
   *
   * @endif
   */
  int SharedMemory::unlink()
  {
	return -1;
  }


  /*!
  * @if jp
  *
  * @brief Lɐς݂̊mF
  *
  *
  *
  * @return true: 쐬ς, false: 쐬
  *
  * @else
  *
  * @brief 
  *
  *
  *
  * @return
  *
  * @endif
  */
  bool SharedMemory::created()
  {
	if (m_handle == NULL)
	{
		return false;
	}
	else
	{
		return true;
	}
  }




}