// -*- C++ -*-
/*!
 * @file LocalServiceBase.h
 * @brief Local service provider base class and its factory
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2011
 *     Noriaki Ando
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

#ifndef RTM_LOCALSERVICEBASE_H
#define RTM_LOCALSERVICEBASE_H

#include <vector>

#include <coil/Mutex.h>
#include <coil/Factory.h>
#include <coil/Timer.h>
#include <coil/Properties.h>

namespace RTM
{
  
  // forward decl
  class LocalServiceBase;
  
  /*!
   * @if jp
   * @brief LocalServiceΥץեǡ
   * @else
   * @brief Profile data structure of LocalService
   * @endif
   */
  class LocalServiceProfile
  {
  public:
    /*!
     * @if jp
     * @brief LocalServiceΥӥ̾
     * @else
     * @brief The name of LocalService
     * @endif
     */
    std::string name;
    /*!
     * @if jp
     * @brief LocalServiceθͭID
     * @else
     * @brief The unique ID of LocalService
     * @endif
     */
    std::string uuid;
    /*!
     * @if jp
     * @brief LocalServiceΥץѥƥ
     * @else
     * @brief Properties of LocalService
     * @endif
     */
    coil::Properties properties;
    /*!
     * @if jp
     * @brief LocalServiceΥݥ
     * @else
     * @brief The pointer to LocalService
     * @endif
     */
    LocalServiceBase* service;
  };
  
  /*!
   * @if jp
   * @brief LocalServiceΥꥹ
   * @else
   * @brief List of LocalServiceProfile
   * @endif
   */
  typedef std::vector<LocalServiceProfile> LocalServiceProfileList;
  
  /*!
   * @if jp
   *
   * @brief LocalService쥯饹
   *
   * Local Service Ȥ RT-MiddlewareΥޥ͡ǡǳƼ掠
   * 󶡤뤿λȤߤǤ롣Local ServiceˤϤĤΥ桼
   * ͤ롣
   *
   * 1. RTCåФƥӥ󶡤롣ݡͥȤγ
   * ȯԤϡФơLocal ServiceΥ󥹥󥹤ؤλȤ
   * ơΥӥѤ뤳ȤǤ롣
   *
   * 2. ޥ͡˵ǽɲä뤿ΥӥȤϡRTC̾
   * ϿCORBA͡ॵӥˡʥǥ쥯ȥꥵ
   * ֥ɥ㥹Ȥˤ̾εǽʤɤ򿷤ʵǽޥ͡
   * ɲä뤿Ѥ뤳ȤǤ롣ޥ͡ˤϡ͡ʥ
   * ФեåѰդƤꡢѤ뤳Ȥǡޥ͡
   * ͡ʥ٥ȤФƥ򵯤ȤǤ롣
   *
   * 3. ޥ͡ϼȤΥӥʳղŪ˳Фƥӥ
   * 󶡤뵡ǽͭ롣CORBAͳǡεǽȤ
   * ХǥХ꥽ΥޥͥȵǽФƥˡ
   * 롣ӥΥ󥹥󥹲ˡޥ͡Фơޥ͡
   * ӥȤϿ뤳ȤǡΥդ뤳Ȥ
   * Ǥ褦ˤʤ롣
   *
   * Local Service ϥ⥸塼Ȥƥɤ졢̾ͣΥ󥹥
   * 롣ʤLocalService ϥ󥰥ȥȤƼ
   * 롣󥹥󥹲줿ӥϥӥꥹȤϿ졢RTC
   * ̾UUIDˤäƥ󥹥󥹤λȤꤹ뤳ȤǤ롣
   *
   * Υ֥ȤΥ饤եϰʲ̤ꡣ
   *
   * -# ֥Ȥ̾֥ͭ (so, DLL) Ȥƥѥ롦
   *    󥯤롣
   * -# ޥ͡Фƥɤȥ⥸塼ؿˤꥪ
   *    ȥեȥ꤬LocalServiceFactory ФϿ롣
   *    ϿΥˤUUID̾Ѥ졢ˤꥵӥ̤
   *    롣
   * -# rtc.confΥե졼ˤꡢͭ뤳Ȥ
   *    ꤵƤ륵ӥץХϡRTCεưƱ˥
   *    󥹲롣
   * -# 󥹥󥹲塢ؿ init() ƤФ롣ˤ
   *    ӥΤΥե졼󥪥ץ coil::Property
   *    Ϥ롣
   * -# ޥ͡㥵ӥڥ졼 reinit ƤФȡ٤Ƥ
   *     Local Service  reinit ƤӽФ졢줿ե졼
   *     ɤ߹ޤ롣
   * -# ޥ͡λˤϡ٤Ƥ Local Service  finalizeƤ
   *       Ф졢ΤΤǡǥ꥽βʤɽλ
   *       
   *
   * Υ饹μäƤϡʤȤʲν貾۴ؿ
   * ɬפ롣
   *
   * - init(): ؿͿ줿 RTObject  ServiceProfile 
   *   顢֥Ȥ롣
   * - reinit(): ƽؿServiceProfile 󹹿ΤƱ
   *   IDǸƤӽФ뤳Ȥͭ뤬κݤˤδؿ
   *   ServiceProfile ȤȤ˸ƤӽФ롣ؿǤϡѹʤ
   *   ƽ롣
   * - getProfile(): ꤵ줿ץե֤ؿ
   * - finalize(): λ󥷥塼ޤǥåݤ˸ƤӽФ
   *   ؿؿǤϽλ롣
   *
   * LocalService ̾ﶦ֥ͭȤȤƥѥ롦
   * 롣֥ͭȤΥȥݥȤ̾拾ѥ뤵줿ե
   * ̾ basename + "Init" ˤƤʲˡ饹̾ե
   * ̾ȥݥȴؿ̾ο侩򼨤
   *
   * - 饹̾: MyLocalService
   * - ե̾: MyLocalService.h. MyLocalService.cpp
   * - ֥ͭ̾: MyLocalService.so (or DLL)
   * - ȥݥȴؿ̾: MyLocalServiceInit()
   *
   * ȥݥȴؿ̾ʲΤ褦ˡLocalServiceFactory
   * 󥷥塼ޤΥեȥ (ȲΥե󥯥) Ͽʲ
   * 褦ʴؿˤʤ롣
   *
   * <pre>
   * extern "C"
   * {
   *   void MyLocalServiceInit()
   *   {
   *     RTC::LocalServiceFactory& factory
   *       = RTC::LocalServiceFactory::instance();
   *     factory.addFactory(::RTC::MyLocalSerivce::name,
   *                        ::coil::Creator< ::RTC::LocalServiceBase,
   *                        ::RTC::MyLocalService>,
   *                        ::coil::Destructor< ::RTC::LocalServiceBase,
   *                        ::RTC::MyLocalService>);
   *   }
   * };
   * </pre>
   *
   * @else
   *
   * @endif
   *
   */
  class LocalServiceBase
  {
  public:
    /*!
     * @if jp
     * @brief ۥǥȥ饯
     * @else
     * @brief virtual destructor
     * @endif
     */
    virtual ~LocalServiceBase()
    {
    }
    
    /*!
     * @if jp
     * @brief LocalService饹νؿ
     *
     * Υ֥ȤνԤLocalService 硢
     * ϡinit()ؿˤͿ롣
     *
     * @param profile Ϳ줿 LocalServiceProfile
     * @return Ϳ줿 LocalServiceProfile ξ false
     *
     * @else
     * @brief Initialization function of the LocalService class
     *
     * @endif
     */
    virtual bool
    init(const coil::Properties& props) = 0;
    /*!
     * @if jp
     * @brief LocalService饹κƽؿ
     *
     * TODO: ɥȺ
     *
     * @param profile Ϳ줿 LocalServiceProfile
     * @return  LocalServiceProfile Ϳ줿 false
     *
     * @else
     * @brief Reinitialization function of the LocalService class
     *
     * TODO: Documentation
     *
     * @endif
     */
    virtual bool
    reinit(const coil::Properties& props) = 0;
    
    /*!
     * @if jp
     * @brief LocalServiceProfile ֤
     *
     * init()/reinit()Ϳ줿 LocalServiceProfile ̾索֥
     * ݻ롣δؿǤݻƤ ServiceProfile 
     * 
     *
     * @return Υ֥ȤݻƤ LocalServiceProfile
     *
     * @else
     * @brief Getting LocalServiceProfile
     *
     * TODO: Documentation
     *
     * @endif
     */
    virtual const LocalServiceProfile&
    getProfile() const = 0;
    
    /*!
     * @if jp
     * @brief λ
     *
     * LocalService λݤ˸ƤӽФ뽪λѴؿ
     * Ȥݻ꥽ʤɤνԤ
     *
     * @else
     * @brief Finalization
     *
     * TODO: Documentation
     *
     * @endif
     */
    virtual void
    finalize() = 0;
  };
  
  /*!
   * @if jp
   * @brief LocalServiceFactory  typedef
   * @else
   * @brief typedef of sdoServiceProviderFactory
   * @endif
   */
  typedef ::coil::GlobalFactory< ::RTM::LocalServiceBase > LocalServiceFactory;
  
#if defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__)
  /*!
   * @if jp
   * @brief 饹ƥץ졼ȤŪ󥹥󥹲
   * @else
   * @brief Explicit instantiation of class template
   * @endif
   */
  EXTERN template class DLL_PLUGIN
  ::coil::GlobalFactory< ::RTM::LocalServiceBase >;
#endif
}; // namespace RTM

#ifdef __GNUC__
extern template class coil::Singleton<coil::GlobalFactory<RTM::LocalServiceBase> >;
#endif

#endif // RTM_LOCALSERVICEBASE_H
